from whistlepy.client.api.base import BaseWhistleAPI


class WhistleOAuthApi(BaseWhistleAPI):

    API_PATH_URL = "api/v2/index.php/user/"

    def auth_user(self, verify):
        """
        根据 verify 验证用户身份
        详情请参考
        http://open.weishao.com.cn/doc/api#yzyh

        :param verify: 通过成员授权获取到的verify
        :param access_token: access_token
        :return: 返回的 JSON 数据包
        """

        return self._get(
            'authUser',
            params={
                'verify': verify,
                'access_token': self._access_token
            }
        )

    def get_user_info(self, student_number):
        """
        根据 student_number 获取用户信息
        详情请参考
        http://open.weishao.com.cn/doc/api#hvyhxx

        :param student_number: 通过成员授权获取到的student_number
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'getInfo',
            params={
                'student_number': student_number,
                'access_token': self._access_token
            }
        )



from whistlepy.client.api.oauth import WhistleOAuthApi
from whistlepy.client.api.notification import WhistleNotificationApi
from whistlepy.client.api.organization import WhistleOrganizationApi
from whistlepy.client.api.geo import WhistleWifiLocApi
from whistlepy.client.api.label import WhistleLabelApi



from enum import Enum

class WhistleErrorCode(Enum):
    """
    微哨接口返回码，全局返回码请参考 http://open.weishao.com.cn/doc/api
    """

    成功 = 1000
    用户未登陆 = 1001
    用户verify不正确 = 1002
    app_key错误或应用未上架 = 1003
    参数不全 = 1004
    用户为订阅该该应用不能向其发送通知 = 1005
    所查信息不存在 = 1006
    未知的消息类型 = 1007
    消息格式错误 = 1008
    节点拥有子节点不能删除 = 1009
    标题过长 = 1010
    消息内容过长 = 1011
    数据已存在 = 1012
    时间格式不对或者定时时间已过 = 1013
    参数domain错误domain不存在 = 1014
    获取学校消息错误= 1015
    ldap服务器连接失败 = 1016
    用户名或密码错 = 1017
    用户已被禁用 = 1018
    oauth服务异常 = 1019
    用户数量大于限定值 = 1020
    学校未开通定位功能 = 2001
    所查位置节点不存在 = 2002
    学号不存在 = 2002
    所查信息不存在哦 = 2003



class BaseWhistleAPI(object):
    """ Whistle API base class """

    API_PATH_URL = ""

    def __init__(self, client):
        self._client = client

    def _get(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL

        return self._client.get(self.API_PATH_URL + url, **kwargs)

    def _post(self, url, **kwargs):
        if getattr(self, 'API_BASE_URL', None):
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(self.API_PATH_URL + url, **kwargs)

    @property
    def _access_token(self):
        return self._client.access_token

    @property
    def session(self):
        return self._client.session

    @property
    def appkey(self):
        return self._client.appkey



from whistlepy.utils import to_binary, to_text


class WhistleException(Exception):
    """Base exception for Whistlepy"""

    def __init__(self, errcode, errmsg):
        """
        :param errcode: Error code
        :param errmsg: Error message
        """
        self.errcode = errcode
        self.errmsg = errmsg

    def __str__(self):
        _repr = 'Error code: {code}, message: {msg}'.format(
            code=self.errcode,
            msg=self.errmsg
        )

        return to_text(_repr)

    def __repr__(self):
        _repr = '{klass}({code}, {msg})'.format(
            klass=self.__class__.__name__,
            code=self.errcode,
            msg=self.errmsg
        )

        return to_text(_repr)


class WhistleClientException(WhistleException):
    """Whistle API client exception class"""
    def __init__(self, errcode, errmsg, client=None,
                 request=None, response=None):
        super(WhistleClientException, self).__init__(errcode, errmsg)
        self.client = client
        self.request = request
        self.response = response


class InvalidSignatureException(WhistleException):
    """Invalid signature exception class"""

    def __init__(self, errcode=-40001, errmsg='Invalid signature'):
        super(InvalidSignatureException, self).__init__(errcode, errmsg)


class APILimitedException(WhistleClientException):
    """Whistle API call limited exception class"""
    pass


class InvalidAppIdException(WhistleException):
    """Invalid app_id exception class"""

    def __init__(self, errcode=-40005, errmsg='Invalid AppId'):
        super(InvalidAppIdException, self).__init__(errcode, errmsg)


class WhistleOAuthException(WhistleClientException):
    """Whistle OAuth API exception class"""
    pass




from whistlepy.client.api.base import BaseWhistleAPI


class WhistleWifiLocApi(BaseWhistleAPI):
    API_PATH_URL = "api/v2/index.php"

    def get_loc_info(self):
        """
        第三方应用服务器通过本方法可以获取在定位引擎中配置的所有位置的信息。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :return: 返回的 JSON 数据包
        """

        return self._get(
            '',
            params={
                'm': "wifiloc",
                'a': "getLocInfo",
                'access_token': self._access_token
            }
        )

    def get_user_list(self, poiList):
        """
        第三方应用服务器通过本方法可以获取在某些位置节点下目前所有用户的信息。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param poiList 查询位置的poiid。

        :return: 返回的 JSON 数据包
        """

        return self._post(
            '?m=wifiloc&a=getUserList&access_token=' + self._access_token,
            params={
                'poiList': poiList,
                'access_token': self._access_token
            }
        )

    def get_user_loc(self, userList):
        """
        第三方应用服务器通过本方法可以查询某些用户的当前位置。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param userList ["1", "2", "3"] //用户学工号的列表。

        :return: 返回的 JSON 数据包
        """

        return self._post(
            '?m=wifiloc&a=getUsersLoc&access_token=' + self._access_token,
            params={
                'userList': userList,
                'access_token': self._access_token
            }
        )

    def get_user_loc_hist(self, student_number, start_ts=None,end_ts=None):
        """
        第三方应用服务器通过本方法可以查询某个用户的历史轨迹，目前支持7天内轨迹查询。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param student_number 用户的学工号
        :param start_ts       查询轨迹的起始unix时间戳，默认为当前时刻时间戳
        :param end_ts         查询轨迹的结束unix时间戳，默认为当前时刻时间戳

        :return: 返回的 JSON 数据包
        """
        params = {
            'm': "wifiloc",
            'a': "getUserLocHist",
            'student_number': student_number,
            'access_token': self._access_token
        }
        if start_ts:
            params["start_ts"] = start_ts

        if end_ts:
            params["end_ts"] = end_ts

        return self._get(
            '',
            params=params
        )

    def set_push_callback(self, callback_url):
        """
        第三方应用服务器如果需要使用位置变化实时推送接口，首先设置服务器回调接口url，
        当订阅该应用的用户位置发生变化时，将会同POST请求方式把实时位置发送到该url。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param callback_url 回调接口的url，参数务必进行url encode

        :return: 返回的 JSON 数据包
        """

        return self._get(
            '',
            params={
                'm': "wifiloc",
                'a': "setPushCallback",
                'callback_url': callback_url,
                'access_token': self._access_token
            }
        )



# -*- coding: utf-8 -*-
"""
    Whistlepy.utils
    ~~~~~~~~~~~~~~~

    This module provides some useful utilities.

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
from __future__ import absolute_import, unicode_literals
import string
import random
import hashlib

try:
    '''Use simplejson if we can, fallback to json otherwise.'''
    import simplejson as json
except ImportError:
    import json  # NOQA

import six
import six.moves.urllib.parse as urlparse


class ObjectDict(dict):
    """Makes a dictionary behave like an object, with attribute-style access.
    """

    def __getattr__(self, key):
        if key in self:
            return self[key]
        return None

    def __setattr__(self, key, value):
        self[key] = value


class WhistleSigner(object):
    """Whistle data signer"""

    def __init__(self, delimiter=b''):
        self._data = []
        self._delimiter = to_binary(delimiter)

    def add_data(self, *args):
        """Add data to signer"""
        for data in args:
            self._data.append(to_binary(data))

    @property
    def signature(self):
        """Get data signature"""
        self._data.sort()
        str_to_sign = self._delimiter.join(self._data)
        return hashlib.sha1(str_to_sign).hexdigest()


def check_signature(token, signature, timestamp, nonce):
    """Check Whistle callback signature, raises InvalidSignatureException
    if check failed.

    :param token: Whistle callback token
    :param signature: Whistle callback signature sent by Whistle server
    :param timestamp: Whistle callback timestamp sent by Whistle server
    :param nonce: Whistle callback nonce sent by Whistle sever
    """
    signer = WhistleSigner()
    signer.add_data(token, timestamp, nonce)
    if signer.signature != signature:
        from Whistlepy.exceptions import InvalidSignatureException

        raise InvalidSignatureException()


def to_text(value, encoding='utf-8'):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return six.binary_type(value)


def timezone(zone):
    """Try to get timezone using pytz or python-dateutil

    :param zone: timezone str
    :return: timezone tzinfo or None
    """
    try:
        import pytz
        return pytz.timezone(zone)
    except ImportError:
        pass
    try:
        from dateutil.tz import gettz
        return gettz(zone)
    except ImportError:
        return None


def random_string(length=16):
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)
    return ''.join(rand_list)


def get_querystring(uri):
    """Get Querystring information from uri.

    :param uri: uri
    :return: querystring info or {}
    """
    parts = urlparse.urlsplit(uri)
    return urlparse.parse_qs(parts.query)


def byte2int(c):
    if six.PY2:
        return ord(c)
    return c



from whistlepy.client.base import BaseWhistleClient
from whistlepy.client import api


class WhistleClient(BaseWhistleClient):
    """
    微信 API 操作类
    通过这个类可以操作微信 API，发送主动消息、群发消息和创建自定义菜单等。
    """

    API_BASE_URL = 'https://api.weishao.com.cn/'

    def __init__(self, appkey, secret, access_token=None,
                 session=None, timeout=None, auto_retry=True, scope="all"):
        super(WhistleClient, self).__init__(
            appkey, access_token, session, timeout, auto_retry
        )
        self.appkey = appkey
        self.secret = secret
        self.scope = scope
        # 插件
        self.oauth = api.WhistleOAuthApi(self)
        self.notification = api.WhistleNotificationApi(self)
        self.org = api.WhistleOrganizationApi(self)
        self.label = api.WhistleLabelApi(self)
        self.geo = api.WhistleWifiLocApi(self)

    def fetch_access_token(self):
        """
        获取/刷新 access token
        详情请参考 http://open.weishao.com.cn/doc/api#hqtoken
        :return: 返回的 JSON 数据包
        """
        return self._fetch_access_token(
            url='https://api.weishao.com.cn/oauth/token',
            data={
                'grant_type': 'client_credentials',
                'app_key': self.appkey,
                'app_secret': self.secret,
                'scope': self.scope
            }
        )



from whistlepy.session import SessionStorage


class ShoveStorage(SessionStorage):

    def __init__(self, shove, prefix='Whistleepy'):
        self.shove = shove
        self.prefix = prefix

    def key_name(self, key):
        return '{0}:{1}'.format(self.prefix, key)

    def get(self, key, default=None):
        key = self.key_name(key)
        try:
            return self.shove[key]
        except KeyError:
            return default

    def set(self, key, value, ttl=None):
        if value is None:
            return

        key = self.key_name(key)
        self.shove[key] = value

    def delete(self, key):
        key = self.key_name(key)
        try:
            del self.shove[key]
        except KeyError:
            pass



import time
import inspect
import logging
import warnings

import six
import requests

from whistlepy.utils import json, get_querystring
from whistlepy.session.memorystorage import MemoryStorage
from whistlepy.exceptions import WhistleClientException, APILimitedException
from whistlepy.client.api.base import BaseWhistleAPI


logger = logging.getLogger(__name__)


def _is_api_endpoint(obj):
    return isinstance(obj, BaseWhistleAPI)


class BaseWhistleClient(object):

    _http = requests.Session()

    API_BASE_URL = ''

    def __new__(cls, *args, **kwargs):
        self = super(BaseWhistleClient, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, appkey, access_token=None, session=None, timeout=None, auto_retry=True):
        self.appkey = appkey
        self.expires_at = None
        self.session = session or MemoryStorage()
        self.timeout = timeout
        self.auto_retry = auto_retry

        if isinstance(session, six.string_types):
            from shove import Shove
            from whistlepy.session.shovestorage import ShoveStorage

            querystring = get_querystring(session)
            prefix = querystring.get('prefix', ['Whistlepy'])[0]

            shove = Shove(session)
            storage = ShoveStorage(shove, prefix)
            self.session = storage

        if access_token:
            self.session.set(self.access_token_key, access_token)

    @property
    def access_token_key(self):
        return '{0}_access_token'.format(self.appkey)

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(('http://', 'https://')):
            api_base_url = kwargs.pop('api_base_url', self.API_BASE_URL)
            url = '{base}{endpoint}'.format(
                base=api_base_url,
                endpoint=url_or_endpoint
            )
        else:
            url = url_or_endpoint

        if 'params' not in kwargs:
            kwargs['params'] = {}
        if isinstance(kwargs['params'], dict) and \
                'access_token' not in kwargs['params']:
            kwargs['params']['access_token'] = self.access_token
        if isinstance(kwargs.get('data', ''), dict):
            body = json.dumps(kwargs['data'], ensure_ascii=False)
            body = body.encode('utf-8')
            kwargs['data'] = body

        if method.lower() == "post":
            kwargs['json'] = kwargs['params']
            del kwargs['params']

        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        result_processor = kwargs.pop('result_processor', None)
        res = self._http.request(
            method=method,
            url=url,
            **kwargs
        )
        print(url)
        return self._handle_result(
            res, method, url, result_processor, **kwargs
        )

    def _decode_result(self, res):
        try:
            result = json.loads(res.content.decode('utf-8', 'ignore'), strict=False)
        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            logger.debug('Can not decode response as JSON', exc_info=True)
            return res
        return result

    def _handle_result(self, res, method=None, url=None,
                       result_processor=None, **kwargs):
        if not isinstance(res, dict):
            # Dirty hack around asyncio based AsyncWhistleClient
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result

        if 'errcode' in result:
            result['errcode'] = int(result['errcode'])
        print(result)

        if 'code' in result and result["code"] == 401:
            if result['errno'] == 100401:
                print("access_token  errors")

        if 'errcode' in result and result['errcode'] != 1000:
            errcode = result['errcode']
            errmsg = result.get('errmsg', errcode)
            if self.auto_retry and errcode in []:
                logger.info('Access token expired, fetch a new one and retry request')
                self.fetch_access_token()
                access_token = self.session.get(self.access_token_key)
                kwargs['params']['access_token'] = access_token
                return self._request(
                    method=method,
                    url_or_endpoint=url,
                    result_processor=result_processor,
                    **kwargs
                )

        return result if not result_processor else result_processor(result)

    def get(self, url, **kwargs):
        return self._request(
            method='get',
            url_or_endpoint=url,
            **kwargs
        )

    def _get(self, url, **kwargs):
        warnings.warn('`_get` method of `WhistleClient` is deprecated, will be removed in 1.6,'
                      'Use `get` instead',
                      DeprecationWarning, stacklevel=2)
        return self.get(url, **kwargs)

    def post(self, url, **kwargs):
        return self._request(
            method='post',
            url_or_endpoint=url,
            **kwargs
        )

    def _post(self, url, **kwargs):
        warnings.warn('`_post` method of `WhistleClient` is deprecated, will be removed in 1.6,'
                      'Use `post` instead',
                      DeprecationWarning, stacklevel=2)
        return self.post(url, **kwargs)

    def _fetch_access_token(self, url, data):
        """ The real fetch access token """
        logger.info('Fetching access token')
        res = self._http.post(
            url=url,
            json=data
        )
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WhistleClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response
            )
        result = res.json()
        if 'errcode' in result and result['errcode'] != 0:
            raise WhistleClientException(
                result['errcode'],
                result['errmsg'],
                client=self,
                request=res.request,
                response=res
            )

        expires_in = 7200
        if 'expires_in' in result:
            expires_in = result['expires_in']
        self.session.set(
            self.access_token_key,
            result['access_token'],
            expires_in
        )
        self.expires_at = int(time.time()) + expires_in
        return result

    def fetch_access_token(self):
        raise NotImplementedError()

    @property
    def access_token(self):
        """ Whistle access token """
        access_token = self.session.get(self.access_token_key)
        if access_token:
            if not self.expires_at:
                # user provided access_token, just return it
                return access_token

            timestamp = time.time()
            if self.expires_at - timestamp > 60:
                return access_token

        self.fetch_access_token()
        return self.session.get(self.access_token_key)



from whistlepy.client.api.base import BaseWhistleAPI


class WhistleNotificationApi(BaseWhistleAPI):
    API_PATH_URL = "api/v2/index.php/notification/"

    def get_list(self):
        """
        第三方应用服务器通过本方法可以查询本应用已经发送过的通知记录.

        详情请参考
        http://open.weishao.com.cn/doc/api#yzyh

        :return: 返回的 JSON 数据包
        """

        return self._get(
            'getList',
            params={
                'access_token': self._access_token
            }
        )

    def get_detail(self, notification_id):
        """
        第三方应用服务器通过本方法可以查询指定通知记录的详情.

        详情请参考
        http://open.weishao.com.cn/doc/api#hvyhxx

        :param notification_id 通知的id
        :return: 返回的 JSON 数据包
        """
        return self._get(
            'getDetails',
            params={
                'notification_id': notification_id,
                'access_token': self._access_token
            }
        )

    def send_notification(self, notification_data):
        """
        第三方应用服务器通过本方法可以发送通知.

        详情请参考
        http://open.weishao.com.cn/doc/api#fstz

        :param notification_data  {type: sendList: .......}
        :return: 返回的 JSON 数据包(notification_id)
        """
        notification_data["access_token"] = self._access_token
        print(notification_data)
        return self._post(
            'sendNotification?access_token=' + self._access_token,
            params=notification_data
        )


from whistlepy.client.api.base import BaseWhistleAPI


class WhistleOrganizationApi(BaseWhistleAPI):
    API_PATH_URL = "api/v2/index.php/organization/"

    def get_org_tree(self, organization_id=0):
        """
        第三方应用服务器通过本方法可以查询给定组织节点下对应的组织信息和成员构成(但不递归子组织节点).

        详情请参考
        http://open.weishao.com.cn/doc/api#zzs

        :param organization_id 组织节点id,可通过查询0节点找到学校的根节点信息
        :param access_token: access_token
        :return: 返回的 JSON 数据包
        """

        return self._get(
            'getOrgTree',
            params={
                'organization_id': organization_id,
                'access_token': self._access_token
            }
        )


from setuptools import setup
from setuptools import setup, find_packages

readme = 'README.md'

with open(readme, 'rb') as f:
    long_description = f.read().decode('utf-8')

setup(
    name='whistlepy',
    version='0.0.6',
    author='sai',
    author_email='3030159@qq.com',
    url='https://github.com/atuple/whistlepy',
    keywords='whistle, SDK, weishao',
    description=u'whistle sdk',
    license='MIT',
    packages=["whistlepy", "whistlepy.client", "whistlepy.session", "whistlepy.client.api"],
    zip_safe=False,
    install_requires=[
        "requests>=2.4.3",
        "python-dateutil>=2.5.2"
    ]
)



from whistlepy.client.api.base import BaseWhistleAPI


class WhistleLabelApi(BaseWhistleAPI):
    API_PATH_URL = "api/v2/index.php/label/"

    def get_list(self):
        """
        第三方应用服务器通过本方法可以查询学校已创建的标签，与获取用户信息接口配合使用，可以区分不同身份属性的用户。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param organization_id 组织节点id,可通过查询0节点找到学校的根节点信息

        :return: 返回的 JSON 数据包
        """

        return self._get(
            'getList',
            params={
                'access_token': self._access_token
            }
        )

    def get_user_list(self, label_id, offset=None, pagesize=None):
        """
        第三方应用服务器通过本方法可以查询学校某标签下的用户列表。

        详情请参考
        http://open.weishao.com.cn/doc/api#bqlbxyhlb

        :param label_id 标签id
        :param offset   起始位置，为空时从第一条开始取(非必填)
        :param pagesize 单次返回结果数量，为空时取至最后一条（非必填）

        :return: 返回的 JSON 数据包
        """
        params = {
            'label_id': label_id,
            'access_token': self._access_token
        }

        if offset:
            params["offset"] = offset

        if pagesize:
            params["pagesize"] = pagesize

        return self._get(
            'getUserList',
            params=params
        )
